@@ -137,15 +137,20 @@ class XGBfnc:
     """
     Builds the binary classifier within a hyper-parameters tuning model.
 
-    :param n_iter: Number of iterations in cross validation for hyper-parameters tuning, defaults to 5
+    :param n_iter: Number of iterations in cross validation for
+      hyper-parameters tuning, defaults to 5
     :type n_iter: int
-    :param n_jobs_cv: Number of parallel jobs running for hyper-parameters tuning, defaults to None
+    :param n_jobs_cv: Number of parallel jobs running for hyper-parameters
+      tuning, defaults to None
     :type n_jobs_cv: int
-    :param n_jobs_xgb: Number of parallel jobs running for training the classifier, defaults to 2
+    :param n_jobs_xgb: Number of parallel jobs running for training the
+      classifier, defaults to 2
     :type n_jobs_xgb: int
-    :param eval_metric: Evaluation metric for training the classifier , defaults to "aucpr"
+    :param eval_metric: Evaluation metric for training the classifier,
+      defaults to "aucpr"
     :type eval_metric: string
-    :param scoring: Scoring metric for hyper-parameters tuning, defaults to "recall"
+    :param scoring: Scoring metric for hyper-parameters tuning, defaults
+      to "recall"
     :type scoring: string
     :param seed: Random number seed, defaults to None
     :type seed: int
@@ -190,7 +195,8 @@ class XGBfnc:
     :param y_pred: Predicted probabilities with the XGBoost classifier.
     :type y_pred: np.array[float]
 
-    :return: Classification for the input array of probabilities which maximies F1 score.
+    :return: Classification for the input array of probabilities which
+      maximies F1 score.
     :rtype: np.array[int]
     """
     prec, recall, thresh = precision_recall_curve(y_orig, y_pred)
@@ -256,9 +262,11 @@ class XGBfnc:
     Plot roc curve, precision-recall curve and confussion matrices for the
     prediction of both models, i.e., without and with structural properties.
 
-    :param a: Predicted probabilities for the model **without** the structural properties of the network.
+    :param a: Predicted probabilities for the model **without** the structural
+      properties of the network.
     :type a: np.array[float]
-    :param b: Predicted probabilities for the model **including** the structural properties of the network.
+    :param b: Predicted probabilities for the model **including** the
+      structural properties of the network.
     :type b: np.array[float]
     :param labels: Labels of both models for the plots.
     :type labels: List[string]
@@ -291,9 +299,11 @@ class XGBfnc:
     Save the evaluation metrics for prediction of both models, i.e., without
     and with structural properties.
 
-    :param a: Predicted probabilities for the model **without** the structural properties of the network.
+    :param a: Predicted probabilities for the model **without** the structural
+      properties of the network.
     :type a: np.array[float]
-    :param b: Predicted probabilities for the model **including** the structural properties of the network.
+    :param b: Predicted probabilities for the model **including** the
+      structural properties of the network.
     :type b: np.array[float]
     :param labels: Labels of both models for the plots.
     :type labels: List[string]
@@ -319,23 +329,26 @@ class XGBfnc:
     :type n_splits: int
     :param seed: Random number seed, defaults to None
     :type seed: int
-    :param n_iter: Number of iterations in cross validation for hyper-parameters tuning, defaults to 5
+    :param n_iter: Number of iterations in cross validation for
+      hyper-parameters tuning, defaults to 5
     :type n_iter: int, optional
-    :param n_jobs_cv: Number of parallel jobs running for hyper-parameters tuning, defaults to None
+    :param n_jobs_cv: Number of parallel jobs running for hyper-parameters
+      tuning, defaults to None
     :type n_jobs_cv: int
-    :param n_jobs_xgb: Number of parallel jobs running for training the classifier, defaults to 2
+    :param n_jobs_xgb: Number of parallel jobs running for training the
+      classifier, defaults to 2
     :type n_jobs_xgb: int
-    :param eval_metric: Evaluation metric for training the classifier , defaults to "aucpr"
+    :param eval_metric: Evaluation metric for training the classifier,
+      defaults to "aucpr"
     :type eval_metric: string
-    :param scoring: Scoring metric for hyper-parameters tuning, defaults to "recall"
+    :param scoring: Scoring metric for hyper-parameters tuning, defaults to
+      "recall"
     :type scoring: string
 
-    :return: Predicted probabilities with the XGBoost classifier.
-    :rtype: np.array[float]
-    :return: Feature importance measured by total gain.
-    :rtype: dict[string->float]
-    :return: Best parameter combination for the classfifier.
-    :rtype: dict[string->float]
+    :return: Predicted probabilities with the XGBoost classifier, feature
+      importance measured by total gain, and best parameter combination for
+      the classfifier.
+    :rtype: Tuple(np.array[float], dict[string->float], dict[string->float])
     """
     # Scores for each fold
     tpr, tnr = list(),list()
@@ -409,13 +422,15 @@ class XGBfnc:
     :type seed: int
     :param log: Flag for logging of the results of the test, default to False
     :type log: bool
-    :param csv: Flag for saving the results of the test in a csv file, defaults to True
+    :param csv: Flag for saving the results of the test in a csv file, defaults
+      to True
     :type csv: bool
 
-    :return: Result of the structural test, True if the the structural properties improve the prediction performance, False otherwise.
-    :rtype: bool
-    :return: Best parameter combination for the classfifier including the structural properties.
-    :rtype: dict[string->float]
+    :return: Result of the structural test (True if the the structural
+      properties improve the prediction performance, False otherwise),
+      predicted labels and best parameter combination for the classfifier
+      including the structural properties.
+    :rtype: Tuple(bool, np.array[Int], dict[string->float])
     """
 
     # prediction without structural properties
@@ -428,6 +443,7 @@ class XGBfnc:
     strc_df = self.df[self.orig_cols+self.strc_cols].copy()
     strc_pred_prob, strc_fimp, strc_params = self.train(strc_df, n_splits=n_splits, seed=seed)
     strc_perf = self.evaluate(self.y, strc_pred_prob)
+    strc_pred_clf = self.opt_threshold(self.y, strc_pred_prob)
     if log: self.print_performance(strc_pred_prob, 'Including topological properties')
 
     self.plot_performance(wo_pred_prob, strc_pred_prob)
@@ -436,4 +452,4 @@ class XGBfnc:
     # test whether the structural properties of the network improve the
     # performance of the prediction
     ans = (wo_perf['tpr'] + 0.05 < strc_perf['tpr']) and (wo_perf['tnr'] < strc_perf['tnr'])
-    return ans, strc_params
+    return ans, strc_pred_clf, strc_params
